package org.confluence.mod.client.renderer.entity.projectile.sword;

import com.mojang.blaze3d.vertex.PoseStack;
import com.mojang.math.Axis;
import net.minecraft.client.model.EntityModel;
import net.minecraft.client.renderer.entity.EntityRendererProvider;
import net.minecraft.resources.ResourceLocation;
import net.minecraft.util.Mth;
import net.minecraft.world.phys.Vec3;
import org.confluence.mod.common.entity.projectile.sword.SwordProjectile;
import org.confluence.terraentity.client.entity.renderer.BaseEntityRenderer;

public class ForwardProjRenderer<T extends SwordProjectile, S extends SwordProjectile, M extends EntityModel<S>> extends BaseEntityRenderer<T, S, M> {
    private final ResourceLocation texture;

    protected boolean rotateZ;
    protected float rotateZSpeed;

    public ForwardProjRenderer(EntityRendererProvider.Context context, M pModel, ResourceLocation texture, float size, float offsetY, float rotateZSpeed) {
        super(context, pModel, size, offsetY);
        this.texture = texture;
        this.rotateZ = rotateZSpeed > 0;
        this.rotateZSpeed = rotateZSpeed;
    }
    public ForwardProjRenderer(EntityRendererProvider.Context context, M pModel, ResourceLocation texture, float size, float offsetY) {
        this(context, pModel, texture, 1 , 0, 0);
    }

    public ForwardProjRenderer(EntityRendererProvider.Context context, M pModel, ResourceLocation texture) {
        this(context, pModel, texture, 1 , 0);
    }

    @Override
    public ResourceLocation getTextureLocation(T swordProjectile) {
        return texture;
    }

    @Override
    public void preRender(T entity, float pEntityYaw, float partialTick, PoseStack poseStack, int packedLight){
        super.preRender(entity, pEntityYaw, partialTick, poseStack, packedLight);
        float f = Math.min((entity.tickCount + partialTick)* 0.1F, 1);
        poseStack.scale(f, f, f);

        Vec3 v = entity.getDeltaMovement();

        float yaw = (float) Math.atan2(v.z, v.x);
        poseStack.mulPose(Axis.YN.rotation(yaw + Mth.HALF_PI));

        if(rotateZ) {
            float pitch = -(float) Math.atan2(v.y, Math.sqrt(v.x * v.x + v.z * v.z));
            poseStack.mulPose(Axis.XN.rotation(pitch));
            poseStack.mulPose(Axis.ZN.rotation((entity.tickCount + partialTick) * rotateZSpeed));
        }


    }
}
