package org.confluence.mod.common.entity.projectile;

import net.minecraft.nbt.CompoundTag;
import net.minecraft.network.protocol.game.ClientboundGameEventPacket;
import net.minecraft.network.syncher.EntityDataAccessor;
import net.minecraft.network.syncher.EntityDataSerializers;
import net.minecraft.network.syncher.SynchedEntityData;
import net.minecraft.server.level.ServerLevel;
import net.minecraft.server.level.ServerPlayer;
import net.minecraft.sounds.SoundEvent;
import net.minecraft.sounds.SoundEvents;
import net.minecraft.util.Mth;
import net.minecraft.world.damagesource.DamageSource;
import net.minecraft.world.entity.Entity;
import net.minecraft.world.entity.EntityType;
import net.minecraft.world.entity.LivingEntity;
import net.minecraft.world.entity.player.Player;
import net.minecraft.world.entity.projectile.AbstractArrow;
import net.minecraft.world.item.ItemStack;
import net.minecraft.world.item.Items;
import net.minecraft.world.item.enchantment.EnchantmentHelper;
import net.minecraft.world.item.enchantment.Enchantments;
import net.minecraft.world.level.Level;
import net.minecraft.world.phys.EntityHitResult;
import net.minecraft.world.phys.Vec3;
import net.neoforged.neoforge.registries.DeferredHolder;
import org.confluence.mod.common.init.ModEntities;
import org.confluence.mod.common.item.bow.BaseArrowItem;
import org.confluence.mod.util.EnchantmentUtil;
import org.confluence.terraentity.hit_effect.EffectStrategy;

import javax.annotation.Nullable;
import java.util.ArrayList;
import java.util.List;
import java.util.function.Consumer;
import java.util.function.Supplier;

public class BaseArrowEntity extends AbstractArrow {

    static final float minSpeedAttackFactor = 0.5f;//速度影响伤害的最小系数

    public static class Tuple {
        public String path;
        public Supplier<Builder> attr;
        Tuple(){}
        public static Tuple create(String path, Supplier<Builder> type){
            Tuple t = new Tuple();
            t.path = path;
            t.attr = type;
            return t;
        }
    }

    private static final EntityDataAccessor<String> TEXTURE_PATH = SynchedEntityData.defineId(BaseArrowEntity.class, EntityDataSerializers.STRING);
    public String texturePath = "";
    private int penetrate = 0;
    private List<LivingEntity> havenBeen = new ArrayList<>();//标记不能重复穿透
    public Builder modify = new Builder();
    private Tuple baseArrowTuple;
    public boolean fullPull = false;

    public BaseArrowEntity(EntityType<? extends AbstractArrow> pEntityType, Level pLevel) {
        super(pEntityType, pLevel);
    }
    /**
     * 自定义箭矢
     * @param owner 发射者
     * @param pickupItemStack 捡起的物品
     * @param firedFromWeapon 发射的武器
     * @param arrow 预定义的箭的类型
     */
    public BaseArrowEntity(LivingEntity owner, ItemStack pickupItemStack, @Nullable ItemStack firedFromWeapon, BaseArrowItem arrow) {
        super(ModEntities.ARROW_PROJECTILE.get(), owner, owner.level(), pickupItemStack, firedFromWeapon);
        this.baseArrowTuple = arrow.getModifier();
        if(baseArrowTuple==null)// 不应该出现这种情况
            this.modify = new Builder();
        else
            this.modify = baseArrowTuple.attr.get();

    }

    /**
     * 自定义箭矢
     * @param owner 发射者
     * @param pickupItemStack 捡起的物品
     * @param firedFromWeapon 发射的武器
     * @param arrow 预定义的箭的类型
     * @param modifyConsumer 属性额外修饰
     */
    public BaseArrowEntity(LivingEntity owner, ItemStack pickupItemStack, @Nullable ItemStack firedFromWeapon, BaseArrowItem arrow, Consumer<Builder> modifyConsumer) {
        this(owner,pickupItemStack,firedFromWeapon, arrow);
        if(modifyConsumer!=null) modifyConsumer.accept(modify);
        if((modify.type & Tag.auto_discard) != 0){
            this.pickup = Pickup.DISALLOWED;
        }
    }


    public void modify(Consumer<Builder> consumer){
        consumer.accept(modify);
    }

    public void onAddedToLevel(){

        if((modify.type & Tag.low_gravity)!=0) this.setNoGravity(true);
        if(baseArrowTuple!=null){
            entityData.set(TEXTURE_PATH,baseArrowTuple.path);
            this.texturePath = baseArrowTuple.path;
        }

        super.onAddedToLevel();
    }

    public void shoot(double pX, double pY, double pZ, float pVelocity, float pInaccuracy) {
        super.shoot(pX, pY, pZ, pVelocity, pInaccuracy);
        this.setDeltaMovement(getDeltaMovement().scale(modify.speedFactor * (level().isRaining() ? modify.speedUpInRain : 1)));
    }

    protected void onHitEntity(EntityHitResult pResult) {
        Entity entity = pResult.getEntity();
        if((modify.type & Tag.cause_fire)!=0)
            entity.setRemainingFireTicks(this.getRemainingFireTicks() + modify.causeFireTick - tickCount);



        if(!(entity instanceof LivingEntity)) {
            super.onHitEntity(pResult);
            return;
        }
        if(havenBeen.contains((LivingEntity) entity)) return;
        float f = (float)this.getDeltaMovement().length();

        Entity entity1 = this.getOwner();
        DamageSource damagesource = this.damageSources().arrow(this, (Entity)(entity1 != null ? entity1 : this));

        // 附魔增伤
        double d0 = this.getBaseDamage();
        if (this.getWeaponItem() != null) {
            Level var9 = this.level();
            if (var9 instanceof ServerLevel) {
                int value = EnchantmentUtil.getEnchantmentLevel(Enchantments.POWER, this.getWeaponItem());
                d0 += value * 0.5f;
            }
        }

        //速度修正系数影响的最小速度值
        f = Math.max(f, minSpeedAttackFactor);
        int i = Mth.ceil(Mth.clamp((double)f * d0, 0.0, 2.147483647E9));

        //暴击增伤
        if (this.isCritArrow()) {
            long j = this.random.nextInt(i / 2 + 2);
            i = (int)Math.min(j + (long)i, 2147483647L);
        }
        // 雨天伤害修正
        float fi = this.level().isRaining()? modify.damageInRain + i : i;

        // 重复穿透
        havenBeen.add((LivingEntity) pResult.getEntity());

        int k = entity.getRemainingFireTicks();
        if(entity.hurt(damagesource, fi + modify.base_damage)){
            this.playSound(getSound(), 1.0F, 1.2F / (this.random.nextFloat() * 0.2F + 0.9F));
            LivingEntity livingentity = (LivingEntity)entity;
            this.doPostHurtEffects(livingentity);
            if (!this.level().isClientSide) {
                livingentity.setArrowCount(livingentity.getArrowCount() + 1);
            }
            //todo 击退
            this.doKnockback(livingentity,damagesource);
            if (modify.knockBack > 0) {
                double d1 = modify.knockBack;
                Vec3 vec3 = this.getDeltaMovement().multiply(1.0D, 0.0D, 1.0D).normalize().scale(0.6 * d1);
                if (vec3.lengthSqr() > 0.0D) {
                    livingentity.push(vec3.x, 0.1D, vec3.z);
                }
            }

            //箭药水效果
            if (!this.level().isClientSide && entity1 instanceof LivingEntity) {
                EnchantmentHelper.doPostAttackEffects((ServerLevel) level(),entity,damagesource);
                //
            }

            //命中自己
            if (livingentity != entity1 && livingentity instanceof Player && entity1 instanceof ServerPlayer && !this.isSilent()) {
                ((ServerPlayer)entity1).connection.send(new ClientboundGameEventPacket(ClientboundGameEventPacket.ARROW_HIT_PLAYER, 0.0F));
            }
            penetrate++;
            if(!canPenetrate()) {
                discard();
                return;
            }

        }else {
            entity.setRemainingFireTicks(k);
            this.setDeltaMovement(this.getDeltaMovement().scale(-0.1D));
            this.setYRot(this.getYRot() + 180.0F);
            this.yRotO += 180.0F;
            if (!this.level().isClientSide && this.getDeltaMovement().lengthSqr() < 1.0E-7D) {
                if (this.pickup == Pickup.ALLOWED) {
                    this.spawnAtLocation(this.getPickupItem(), 0.1F);
                }
            }
        }
    }

    @Override
    public double getBaseDamage() {
        return super.getBaseDamage();
    }

    @Override
    protected void doPostHurtEffects(LivingEntity pLiving) {
        if(getOwner() instanceof LivingEntity owner){
            modify.onHitEffect.forEach(m->m.get().getEffect().accept(owner,pLiving));
            if(fullPull) modify.fullPullHitEffect.forEach(m->m.get().getEffect().accept(owner,pLiving));
        }
        super.doPostHurtEffects(pLiving);
    }

    private static SoundEvent getSound(){
        return SoundEvents.TRIDENT_HIT_GROUND;
    }


    @Override
    protected ItemStack getDefaultPickupItem() {
        // 构造延迟
        if(this.modify == null)
            return Items.ARROW.getDefaultInstance();
        return Items.ARROW.getDefaultInstance();
    }


    @Override
    public void tick(){

        if(!level().isClientSide && tickCount> modify.auto_discard_tick)discard();
        //todo 重力调整
        if((modify.type & Tag.low_gravity)!=0) this.addDeltaMovement(new Vec3(0, modify.gravity_count, 0));

        super.tick();
    }


    public boolean canPenetrate(){
        return penetrate+1 <= modify.penetration_count;
    }

    @Override
    public void defineSynchedData(SynchedEntityData.Builder pBuilder){
        super.defineSynchedData(pBuilder);
        pBuilder.define(TEXTURE_PATH,"");
    }
    @Override
    public void onSyncedDataUpdated(EntityDataAccessor<?> pKey) {
        super.onSyncedDataUpdated(pKey);
        this.texturePath =entityData.get(TEXTURE_PATH);

    }

    @Override
    public void addAdditionalSaveData(CompoundTag tag) {
        super.addAdditionalSaveData(tag);
    }
    @Override
    public void readAdditionalSaveData(CompoundTag tag) {
        super.readAdditionalSaveData(tag);
        if(getPickupItem().getItem() instanceof BaseArrowItem arrow && arrow.getModifier() != null) {
            texturePath = arrow.getModifier().path;
            entityData.set(TEXTURE_PATH,texturePath);
        }
    }

    @Override
    protected float getWaterInertia() {
        return modify.speedUpInWater * super.getWaterInertia();
    }




    /** 能力表 **/
    public static class Tag{
        public static final int penetration = 1;//可穿透
        public static final int low_gravity = 2;//低重力
        public static final int auto_discard = 4;//超过时间自动消失
        public static final int cause_fire = 8;//火焰附加

    }
    public static class Builder{
        private int type = 0;
        public int penetration_count = 0;
        private int gravity_count = 0;
        private int auto_discard_tick = 1200;
        public float base_damage = 2;
        private float speedFactor = 1;
        private float knockBack = 0;
        private int causeFireTick = 0;

        // 雨天增益
        private float speedUpInRain = 1;
        private float speedUpInWater = 1;
        private float damageInRain = 0;

        private BaseArrowItem transformArrow = null;
        public final List<DeferredHolder<EffectStrategy,EffectStrategy>> onHitEffect = new ArrayList<>();
        public final List<DeferredHolder<EffectStrategy,EffectStrategy>> fullPullHitEffect = new ArrayList<>();


        public Builder addFullPullHitEffect(DeferredHolder<EffectStrategy,EffectStrategy> consumer){
            this.fullPullHitEffect.add(consumer);
            return this;
        }
        public Builder addOnHitEffect(DeferredHolder<EffectStrategy,EffectStrategy> consumer){
            this.onHitEffect.add(consumer);
            return this;
        }
        public Builder setDamage(float damage){//基本伤害
            base_damage = damage;
            return this;
        }
        public Builder setKnockBack(float factor){//击退修正系数
            this.knockBack = factor;
            return this;
        }
        public Builder setSpeedUpInRain(float speedUp){//水下和雨天加速
            this.speedUpInRain = speedUp;
            return this;
        }
        public Builder setSpeedInertiaInWater(float speedUp){//水下加速
            this.speedUpInWater = speedUp;
            return this;
        }
        public Builder setDamageInRain(float damage){//水下和雨天伤害修正
            this.damageInRain = damage;
            return this;
        }

        public Builder setPenetration(int count){//穿透次数
            type|= Tag.penetration;
            penetration_count = count;
            if(count > 1){
                type|= Tag.auto_discard;
                auto_discard_tick = Math.min(auto_discard_tick, 120);
            }
            return this;
        }
        public Builder setGravity(int gravity){//重力
            type|= Tag.low_gravity;
            gravity_count = gravity;
            return this;
        }
        public Builder setAutoDiscard(int tick){//消失tick
            type|= Tag.auto_discard;
            auto_discard_tick = tick;
            return this;
        }
        public Builder setSpeedFactor(float factor){//初始速度修正系数
            speedFactor = factor;
            return this;
        }
        public Builder setCauseFire(int tick){//初始火焰增加
            type|= Tag.cause_fire;
            this.causeFireTick = tick;
            return this;
        }
        public Builder setTransformArrow(BaseArrowItem arrow){
            this.transformArrow = arrow;
            return this;
        }
        public int getType(){
            return type;
        }

        @Nullable
        public BaseArrowItem getTransformArrow() {
            return transformArrow;
        }
    }


}
