package org.confluence.mod.mixin.client.renderer;

import com.llamalad7.mixinextras.injector.wrapoperation.Operation;
import com.llamalad7.mixinextras.injector.wrapoperation.WrapOperation;
import com.llamalad7.mixinextras.sugar.Local;
import com.mojang.blaze3d.vertex.PoseStack;
import com.mojang.blaze3d.vertex.VertexConsumer;
import net.minecraft.client.renderer.block.ModelBlockRenderer;
import net.minecraft.client.renderer.block.model.BakedQuad;
import net.minecraft.core.BlockPos;
import net.minecraft.world.level.BlockAndTintGetter;
import net.minecraft.world.level.block.Blocks;
import net.minecraft.world.level.block.state.BlockState;
import org.confluence.mod.client.handler.ClientPacketHandler;
import org.confluence.mod.client.textures.LocalBrushData;
import org.confluence.mod.common.data.saved.BrushData;
import org.confluence.mod.util.ClientUtils;
import org.spongepowered.asm.mixin.Mixin;
import org.spongepowered.asm.mixin.injection.At;

@Mixin(ModelBlockRenderer.class)
public abstract class ModelBlockRendererMixin {
    /**
     * @see org.confluence.mod.mixin.integration.sodium.BlockRendererMixin
     */
    @WrapOperation(method = "putQuadData", at = @At(value = "INVOKE", target = "Lcom/mojang/blaze3d/vertex/VertexConsumer;putBulkData(Lcom/mojang/blaze3d/vertex/PoseStack$Pose;Lnet/minecraft/client/renderer/block/model/BakedQuad;[FFFFF[IIZ)V"))
    private void putColor(VertexConsumer instance, PoseStack.Pose pose, BakedQuad quad, float[] brightness, float red, float green, float blue, float alpha, int[] lightmap, int packedOverlay, boolean readAlpha, Operation<Void> original, @Local(argsOnly = true) BlockPos pPos) {
        int color = LocalBrushData.getColor(pPos, quad.getDirection());
        if (color == BrushData.EMPTY_COLOR || color == BrushData.ECHO_COLOR) {
            original.call(instance, pose, quad, brightness, red, green, blue, alpha, lightmap, packedOverlay, readAlpha);
        } else if (color == BrushData.ILLUMINANT_COLOR) {
            original.call(instance, pose, quad, brightness, red, green, blue, alpha, ClientUtils.FULL_BRIGHT, packedOverlay, readAlpha);
        } else if (color == BrushData.NEGATIVE_COLOR) {
            if (red != 1.0F || green != 1.0F || blue != 1.0F) {
                original.call(instance, pose, quad, brightness, 1.0F - red, 1.0F - green, 1.0F - blue, alpha, lightmap, packedOverlay, readAlpha);
            } else {
                original.call(instance, pose, quad, brightness, red, green, blue, alpha, lightmap, packedOverlay, readAlpha);
            }
        } else {
            float r = (float) (color >> 16 & 255) * ClientUtils.INV_255;
            float g = (float) (color >> 8 & 255) * ClientUtils.INV_255;
            float b = (float) (color & 255) * ClientUtils.INV_255;
            original.call(instance, pose, quad, brightness, r, g, b, alpha, lightmap, packedOverlay, readAlpha);
        }
    }

    @Mixin(targets = "net.minecraft.client.renderer.block.ModelBlockRenderer$AmbientOcclusionFace")
    public static class AmbientOcclusionFaceMixin {
        /**
         * @see org.confluence.mod.mixin.integration.sodium.LightDataAccessMixin
         */
        @WrapOperation(method = "calculate", at = @At(value = "INVOKE", target = "Lnet/minecraft/world/level/BlockAndTintGetter;getBlockState(Lnet/minecraft/core/BlockPos;)Lnet/minecraft/world/level/block/state/BlockState;"))
        private BlockState replace(BlockAndTintGetter instance, BlockPos blockPos, Operation<BlockState> original, @Local(argsOnly = true) BlockPos pos) {
            if (!ClientPacketHandler.hasEchoVisible() && !blockPos.equals(pos) && LocalBrushData.hasEcho(blockPos)) {
                return Blocks.AIR.defaultBlockState();
            }
            return original.call(instance, blockPos);
        }
    }
}
